// Program to demonstrate using one-dim. arrays with functions.
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;

// Global Constant
const int SIZE = 100;

// Prototypes
int locateLargestScore(double scores[], int count);
void getScoresInteractively(double scores[], int & count);
void printScores(double scores[], int count);
double averageScores(double scores[], int count);
void saveScoresToFile(char fileName[], double scores[], int count);
void readScoresFromFile(char fileName[], double scores[], int & count);

int main() {
	double scores[SIZE];
	char fileName[SIZE], response[SIZE];
	int numberOfScores;
	double averageScore, minScore, maxScore;
	
	while (true) {
		cout << "Would you like to read scores from a (f)ile or (i)nteractively (f or i)?: ";
		cin.getline(response, SIZE);
		if (strcmp(response, "f") == 0) {
			cout << "Enter the file name containing the scores: ";
			cin.getline(fileName, SIZE);
			readScoresFromFile(fileName, scores, numberOfScores);
			break;
		} else if (strcmp(response, "i") == 0) {
			getScoresInteractively(scores, numberOfScores);
			break; 
		} else {
			cout << "Invalid response!  Please enter only f or i. " << endl;
		} // end if
	} // end while

	printScores(scores, numberOfScores);

	maxScore = scores[locateLargestScore(scores, numberOfScores)];
	cout << "The largest score is " << maxScore << endl;

	averageScore = averageScores(scores, numberOfScores);
	cout << "The average score is " << fixed << setprecision(1) << averageScore << endl;

	while (true) {
		cout << "\nWould you like to save the scores to a file (y or n)?: ";
		cin.getline(response, SIZE);
		if (strcmp(response, "n") == 0) {
			cout << "Scores not saved" << endl;
			break;
		} else if (strcmp(response, "y") == 0) {
			cout << "Enter the file name where you want the scores saved: ";
			cin.getline(fileName, SIZE);
			saveScoresToFile(fileName, scores, numberOfScores);
			break; 
		} else {
			cout << "Invalid response!  Please enter only f or i. " << endl;
		} // end if
	} // end while

} // end main


//This function returns the index of the largest score in the array scores.
int locateLargestScore(double scores[], int count) {
	int indexOfLargest;
	int indexToTest;

	indexOfLargest = 0;
	for(indexToTest = 1; indexToTest < count; indexToTest++) {
		if (scores[indexToTest] > scores[indexOfLargest]) {
			indexOfLargest = indexToTest;
		} // end if
	} // end for
	return indexOfLargest;
} // end locateLargestScore

// This function interactively reads scores into the scores array 
// until a negative score is read as a sentinel value.  The count returns
// the number of scores read.
void getScoresInteractively(double scores[], int & count) {
	double score;

	count = 0;
	while (true) {  // infinite loop
		cout << "Enter a score (or a negative value when done): ";
		cin >> score;
		cin.ignore(SIZE,'\n');
		if (score < 0) {
			return;
		} else {
			scores[count] = score;
			count++;
		} // end if
	} // end while
} // end getScoresInteractively


// Prints the scores to cout
void printScores(double scores[], int count) {
	int index;

	cout << "There are " << count << " scores.  The scores are: " << endl;
	for (index = 0; index < count; index++) {
		cout << scores[index] << endl;
	} // end for
} // end printScores


// Returns the average of the scores
double averageScores(double scores[], int count) {
	double total;
	int index;

	total = 0;
	for(index = 0; index < count; index++) {
		total += scores[index];
	} // end for

	if (count <= 0) {
		cout << "Error: To calculate the average, you must have one or more scores!" << endl;
		exit(EXIT_FAILURE);
	} else {
		return total / count;
	} // end if

} // end averageScores


// Saves the scores to the specified fileName
void saveScoresToFile(char fileName[], double scores[], int count) {
	int index;
	ofstream outFile;

	outFile.open(fileName);
	for (index = 0; index < count; index++) {
		outFile << scores[index] << endl;
	} // end for
	outFile.close();
} // end saveScoresToFile


// Reads the scores from the specified fileName.
void readScoresFromFile(char fileName[], double scores[], int & count) {
	double score;
	ifstream inFile;

	inFile.open(fileName);
	if (!inFile) {
		cout << "Error opening file <" << fileName << "> for reading!";
	} else {
		count = 0;
		while (inFile >> score) {
			cout << "Echo of score read from file " << score << endl;
			scores[count] = score;
			count++;			
		} // end while
		inFile.close();
	} // end if
} // end readScoresFromFile
